# coding=utf-8

"""Checkers applied on several types of source files"""

import os
import os.path as osp
import tempfile
from subprocess import PIPE, CalledProcessError, check_call

from aslint.base_checkers import (CheckList, DirnameCat, GenericMsg, Report,
                                  TextMsg, check_filename,
                                  checkers_from_context)
from aslint.config import ASCFG
from aslint.i18n import _
from aslint.logger import logger


class ErrorMessages(DirnameCat, GenericMsg):

    """Unused or missing messages
    All error messages must be called in a source file and all messages called
    must exist."""
    id = "C9003"
    fmt = "%(id)s: %(label)s"

    def search(self, srcdir):
        """Check encoding of txt"""
        result = []
        # only in 'src'
        if not osp.isdir(osp.join(srcdir, 'bibpyt', 'Messages')):
            return result
        try:
            unused, not_found = check_messages(srcdir)
            result.extend([': %s (unused)' % msg for msg in unused])
            result.extend([': %s (missing)' % msg for msg in not_found])
        except ValueError as exc:
            result.append(': check failed: %s' % str(exc))
        return result


class Supv002aRequirement(DirnameCat, GenericMsg):

    """Requirements missed to check spelling of messages (supv002a will fail)
    The testcase supv002a checks the spelling of error messages.
    It requires ``aspell`` and the dictionnaries
    ``<DEVTOOLS>/share/spell/code_aster_dict.aspell.per`` and
    ``code_aster_cata.aspell.per``.
    """
    id = "W9004"
    fmt = "%(id)s: %(label)s"

    def search(self, srcdir):
        """Check pre-requisites for supv002a"""
        share = osp.join(ASCFG.get('devtools_root'), 'share', 'spell')
        fdict = osp.join(share, 'code_aster_dict.aspell.per')
        if not osp.isfile(fdict):
            return [": french dictionnary is missing: %s" % fdict]
        fdict = osp.join(share, 'code_aster_cata.aspell.per')
        if not osp.isfile(fdict):
            return [": dictionnary of keywords is missing: %s (see help)"
                    % fdict]
        try:
            check_call(['aspell', '--version'], stdout=PIPE)
        except CalledProcessError:
            return [": 'aspell' not found, not in $PATH"]
        return []


CHECK_LIST = checkers_from_context(globals(), TextMsg)


def check_messages(srcdir):
    """Check the errors messages"""
    from aslint.messages import MessagesManager, CataMessageError
    tmpcache = tempfile.NamedTemporaryFile(prefix='cachedict.').name
    catapy = osp.abspath(osp.join(srcdir, 'code_aster'))
    if not osp.exists(catapy):
        catapy = osp.abspath(osp.join(srcdir, 'catapy'))
    # repref is not necessary because fort, pyt and capy are absolute paths
    msgman = MessagesManager(repref="__unused__",
                             fort=osp.abspath(osp.join(srcdir, 'bibfor')),
                             pyt=osp.abspath(osp.join(srcdir, 'bibpyt')),
                             capy=catapy,
                             cache_dict=tmpcache,
                             verbose=False, force=True)
    msgman.read_cata()
    try:
        unused, not_found = msgman.check_cata()
    except CataMessageError as msg:
        logger.error(_("error during check of error messages:\n%s"), str(msg))
        raise ValueError
    finally:
        try:
            os.remove(tmpcache)
        except OSError:
            pass
    return unused, not_found


def check_global(srcdir):
    """Check a directory. Return a report of the errors."""
    checklist = CheckList()
    checklist.register(CHECK_LIST)
    report = Report()
    logger.info(_("add checking of error messages"))
    lmsg = check_filename(srcdir, checklist.on_dirname())
    report.set(srcdir, lmsg)
    return report
